import {
  mergeData as mergeFunctionalData
} from 'vue-functional-data-merge'
import {
  renderRule as renderFelaRule,
  renderRules as renderFelaRules
} from 'vue-fela'

const HEX_3 = /^#?([a-f\d])([a-f\d])([a-f\d])$/i
const HEX_6 = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i

export const test = (regex) =>
  (value) => regex.test(value)

export const complement = (fn) =>
  (value) => !fn(value)

export const contains = (values) =>
  (value) => values.indexOf(value) !== -1

export const validate = contains

export const ifElse = (condition) =>
  (then, otherwise) => (condition ? then : otherwise)

export const isMeta = test(/^meta$/)

export const isHeading = test(/^h[1-6]$/)

export const isType = (type) => (value) => typeof value === type

export const isNull = (value) => value === null

export const isRegex = (value) => value instanceof RegExp

export const isNumber = isType('number')

export const isString = isType('string')

export const isFunction = isType('function')

export const isUndefined = isType('undefined')

export const isDefined = complement(isUndefined)

export const isLinkActive = (linkPath, routePath, exact) =>
  (exact ? routePath === linkPath : routePath.indexOf(linkPath) === 0)

export const offset = (outer, inner) => (outer - inner) * 0.5

export const units = (count = 1) => 8 * count

const expandRgb = (m, r, g, b) => r + r + g + g + b + b

export const hexToRgbMap = (hex) => {
  const value = HEX_3.test(hex) ? hex.replace(HEX_3, expandRgb) : hex
  const match = HEX_6.exec(value)
  return match ? {
    r: parseInt(match[1], 16),
    g: parseInt(match[2], 16),
    b: parseInt(match[3], 16)
  } : null
}

export const hexToRgb = (hex) => {
  const { r, g, b } = hexToRgbMap(hex)
  return `rgb(${r},${g},${b})`
}

export const hexToRgba = (hex, alpha) => {
  const { r, g, b } = hexToRgbMap(hex)
  return `rgba(${r},${g},${b},${alpha})`
}

export const className = (ctx, name, classes) => {
  return ctx.injections.croudClassName(name, classes)
}

export const mergeData = (ctx, data) => {
  return mergeFunctionalData(ctx.data, data)
}

export const renderRule = (context, rule, props) => {
  const renderer = context.injections.fela
  const state = props ? props : context.props
  return renderFelaRule(renderer, rule, state)
}

export const renderRules = (context, rules, props) => {
  const renderer = context.injections.fela
  const state = props ? props : context.props
  return renderFelaRules(renderer, rules, state)
}

export const camelToKebab = (string) => string
  .replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase()

export const kebabToCamel = (string) => string
  .replace(/(-\w)/g, (m) => m[1].toUpperCase())

export const capitalize = (string) => string
  .charAt(0).toUpperCase().concat(string.slice(1).toLowerCase())

export const wrapKey = (key, prefix, postfix) => {
  let result = key
  if (prefix) result = `${prefix}-${result}`
  if (postfix) result = `${result}-${postfix}`
  return kebabToCamel(result)
}

export const parseEdges = (edges) =>
  (isString(edges) ? edges.split('|') : edges)

export const attempt = (fn, ...args) => {
  if (isFunction(fn)) fn(...args)
}

export const noop = () => null
